IGENOMES_REFERENCES = {
    "TAIR10": "Arabidopsis thaliana (TAIR10)",
    "EB2": "Bacillus subtilis 168 (EB2)",
    "UMD3.1": "Bos taurus (UMD3.1)",
    "bosTau8": "Bos taurus (bosTau8)",
    "WBcel235": "Caenorhabditis elegans (WBcel235)",
    "ce10": "Caenorhabditis elegans (ce10)",
    "CanFam3.1": "Canis familiaris (CanFam3.1)",
    "canFam3": "Canis familiaris (canFam3)",
    "GRCz10": "Danio rerio (GRCz10)",
    "danRer10": "Danio rerio (danRer10)",
    "BDGP6": "Drosophila melanogaster (BDGP6)",
    "dm6": "Drosophila melanogaster (dm6)",
    "EquCab2": "Equus caballus (EquCab2)",
    "equCab2": "Equus caballus (equCab2)",
    "EB1": "Escherichia coli K 12 DH10B (EB1)",
    "Galgal4": "Gallus gallus (Galgal4)",
    "Gm01": "Glycine max (Gm01)",
    "GRCh37": "Homo sapiens (GRCh37)",
    "GRCh38": "Homo sapiens (GRCh38)",
    "hg18": "Homo sapiens (hg18)",
    "hg19": "Homo sapiens (hg19)",
    "hg38": "Homo sapiens (hg38)",
    "Mmul 1": "Macaca mulatta (Mmul 1)",
    "GRCm38": "Mus musculus (GRCm38)",
    "mm10": "Mus musculus (mm10)",
    "IRGSP-1.0": "Oryza sativa japonica (IRGSP-1.0)",
    "CHIMP2.1.4": "Pan troglodytes (CHIMP2.1.4)",
    "panTro4": "Pan troglodytes (panTro4)",
    "Rnor 5.0": "Rattus norvegicus (Rnor 5.0)",
    "Rnor 6.0": "Rattus norvegicus (Rnor 6.0)",
    "rn6": "Rattus norvegicus (rn6)",
    "R64-1-1": "Saccharomyces cerevisiae (R64-1-1)",
    "sacCer3": "Saccharomyces cerevisiae (sacCer3)",
    "EF2": "Schizosaccharomyces pombe (EF2)",
    "Sbi1": "Sorghum bicolor (Sbi1)",
    "Sscrofa10.2": "Sus scrofa (Sscrofa10.2)",
    "susScr3": "Sus scrofa (susScr3)",
    "AGPv3": "Zea mays (AGPv3)"
}

PROCESSES_PATH_S3 = "s3://<RESOURCES_BUCKET>/process"
